#include "CHMpreCompHead.h"
#pragma hdrstop

#ifdef CHM_NO_PRECOMPILED_HEADERS
#  include <CHM/CHMwrapper.h>
#  include <CHM/CHMstring.h>
#  include <CHM/CHMtransport.h>
#  include <CHM/CHMengineClass.h>
#endif

#include <CHM/CHMvector.h>
class CHMwrapperPrivate
{
public:
   CHMwrapperPrivate() {}
   ~CHMwrapperPrivate() {}

   CHMvector<CHMstring>  Buffer;
   CHMengine*            pEngine;
};

CHMwrapper::CHMwrapper()
{
   pMember = new CHMwrapperPrivate;
}

CHMwrapper::~CHMwrapper()
{
   delete pMember;
}

size_t CHMwrapper::ReceiveData
(
   const CHMstring& Data,
   size_t TransportIndex 
)
{
   CHM_ASSERT(pMember != NULL);
   if (TransportIndex >= pMember->Buffer.size())
   {
      pMember->Buffer.resize(TransportIndex + 1);
   }
   pMember->Buffer[TransportIndex] = pMember->Buffer[TransportIndex] + Data;

   CHMstring Message;
   size_t CountOfMessage = 0;
   while (DetectMessage(pMember->Buffer[TransportIndex], Message))
   {
      CountOfMessage++;
      OnMessage(Message, TransportIndex);
   }
   return CountOfMessage;
}

void CHMwrapper::SendResponseMessage
(
   const CHMstring& MessageData,
   size_t           TransportIndex
)
{
   if (Transport() != NULL)
   {
      CHMstring WrappedData(MessageData);
      AddWrapper(WrappedData);
      Transport()->SendResponseMessage(WrappedData, TransportIndex);
   }
}

void CHMwrapper::OnMessage
(
   const CHMstring& MessageData,
   size_t           TransportIndex
)
{
   CHM_ASSERT(pMember != NULL);
   CHM_ASSERT(pMember->pEngine != NULL);
   pMember->pEngine->ParseMessage(MessageData.c_str(), TransportIndex);
}

void CHMwrapper::SetEngine(CHMengine* pEngine)
{
   CHM_ASSERT(pMember != NULL);
   pMember->pEngine = pEngine;
}

CHMengine* CHMwrapper::Engine()
{
   CHM_ASSERT(pMember != NULL);
   return pMember->pEngine;
}

CHMtransport* CHMwrapper::Transport()
{
   CHM_ASSERT(pMember->pEngine != NULL);
   return pMember->pEngine->Transport();
}

void CHMwrapper::ClearData(size_t TransportIndex)
{
   CHM_ASSERT(pMember != NULL);
   if (TransportIndex >= pMember->Buffer.size())
   {
      pMember->Buffer.resize(TransportIndex + 1);
   }
   pMember->Buffer[TransportIndex] = "";   
}
