using System;
using iNTERFACEWARE.Chameleon;

class ConsoleServer : Log
{
    private static string MESSAGE_DEFINITION_FILE = "..\\..\\project.vmd";
    private DynamicEngine m_engine; 

    public ConsoleServer()
    {
        m_engine = new DynamicEngine();   
    }
    public void StartServer() 
    {
        try
        {        
            m_engine.AddLog(this);
            m_engine.Load(MESSAGE_DEFINITION_FILE);
            // comment the following line for strict segment grammar validation
            m_engine.RejectBadSegmentGrammar = false;

            // instantiate a table object
            Table messageTable = new Table();
            Println("This example shows how one can avoid using stub code in Chameleon");
            Println("and instead use a dynamic interface to extract data from an HL7 message.");
            string flatWire;
            flatWire = "MSH|^~\\&|REGADT|MCM|RSP1P8|MCM|199601051530|SEC|ADT^A44|00000007|P|2.3\r";
            flatWire += "EVN|A44|199601051530\r";
            flatWire += "PID|||MR2^^^XYZ||JONES^WILLIAM^A^JR||19501010|M|||123 EAST STREET^^NY^NY^10021||(212)111-3333|||S||ACCT1\r";
            flatWire += "MRG|MR1^^^XYZ||ACCT1\r";
            flatWire += "MRG|MR1^^^XYZ||ACCT1\r";
            flatWire += "MRG|MR1^^^XYZ||ACCT1\r";
         
            int messageIndex = m_engine.ParseMessageDynamically(flatWire, messageTable);
            Println("Got a " + m_engine.MessageName(messageIndex));

            ExamineTable("", messageTable);

            Println("Press Enter to Exit...");
            System.Console.ReadLine();
        }
        catch (ChameleonException ex)
        {
            Println(ex.ToString());
        }
    }

    public void ExamineTable(string indent, Table aTable)
    {
        Println(indent + "Table Name:" + aTable.TableName());
        Println(indent + " count of row = " + aTable.CountOfRow());
        for (int rowIndex = 0;
            rowIndex < aTable.CountOfRow();
            rowIndex++)
        {
            for (int subTableIndex = 0;
                subTableIndex < aTable.CountOfSubTable();
                subTableIndex++)
            {
                ExamineTable(indent + "   ", aTable.SubTable(rowIndex, subTableIndex));
            }
            for (int columnIndex = 0;
                columnIndex < aTable.CountOfColumn();
                columnIndex++)
            {
                string columnOutput;
                columnOutput = indent + aTable.ColumnName(columnIndex) + ": ";
                switch (aTable.ColumnType(columnIndex))
                {
                    case DataType.StringType:
                        columnOutput += aTable.GetString(columnIndex, rowIndex);
                        break;
                    case DataType.DoubleType:
                        columnOutput += aTable.GetDouble(columnIndex, rowIndex);
                        break;
                    case DataType.IntegerType:
                        columnOutput += aTable.GetInteger(columnIndex, rowIndex);
                        break;
                    case DataType.DateTimeType:
                        columnOutput += aTable.GetDate(columnIndex, rowIndex);
                        break;
                }
                Println(columnOutput);
            }
        }
    }
   
    //Log interfase
    public void Println(string output_text) 
    {
        System.Console.WriteLine(output_text);		
    }
   
    public static void Main()
    {
        ConsoleServer Server = new ConsoleServer();
        Server.StartServer();
    }
}
